#include "DebugDrawer.h"
#include <iostream>
#include <GL/glew.h>
#include <assimp\mesh.h>
//#include "Shader.hpp"
#include <glm\gtc\type_ptr.hpp>

using namespace cgue;

DebugDrawer::DebugDrawer(Shader* shader, Camera* c) : debugMode(0) {
	
	wireframeShader = new Shader("Shader/wireframe.vert", "Shader/wireframe.frag");
	camera = c;

	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);
	
	
	float vertices_position[6] = {

	0.0f, 0.0f, 0.0f,
	0.0f, 0.0f, 0.0f,
	

};
	/*
	GLfloat vertices_position[36] = {
		0.0, 0.0, 0.0,
		0.5, 0.0, 0.0,
		0.5, 0.5, 0.0,
		
		0.0, 0.0, 0.0,
		0.0, 0.5, 0.0,
		-0.5, 0.5, 0.0,
		
		0.0, 0.0, 0.0,
		-0.5, 0.0, 0.0,
		-0.5, -0.5, 0.0,

		0.0, 0.0, 0.0,
		0.0, -0.5, 0.0,
		0.5, -0.5, 0.0
	};
	*/

	// Create a Vector Buffer Object that will store the vertices on video memory
	//GLuint vbo;
	glGenBuffers(1, &vbo);

	// Allocate space and upload the data from CPU to GPU
	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, sizeof(vertices_position), vertices_position, GL_STATIC_DRAW);


	//GLuint shaderProgram = create_program("shaders/vert.shader", "shaders/frag.shader");

	// Get the location of the attributes that enters in the vertex shader
	GLint position_attribute = glGetAttribLocation(wireframeShader->programHandle, "position");

	// Specify how the data for position can be accessed
	glVertexAttribPointer(position_attribute, 3, GL_FLOAT, GL_FALSE, 0, 0);

	// Enable the attribute
	glEnableVertexAttribArray(position_attribute);

	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
}

void DebugDrawer::drawLine(const btVector3& from,const btVector3& to,const btVector3& color) {
	
	//camera.offsetPosition(glm::vec3(0.001,0.001,0.001));
	wireframeShader->useShader();


	float vertices_position[6] = {

	from.getX(), from.getY(), from.getZ(),
	to.getX(), to.getY(), to.getZ(),
	

};
	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, sizeof(vertices_position), vertices_position, GL_STATIC_DRAW);

	glUniformMatrix4fv(
		glGetUniformLocation(wireframeShader->programHandle, "projectionMatrix"), 1,
		GL_FALSE, glm::value_ptr(camera->matrix()));
	glBindVertexArray(vao);
	glUniformMatrix4fv(glGetUniformLocation(wireframeShader->programHandle, "modelMatrix"), 1, GL_FALSE, glm::value_ptr(glm::mat4()));

	glDrawArrays(GL_LINES, 0, 2);
}

void DebugDrawer::drawContactPoint(const btVector3& PointOnB,const btVector3& normalOnB,btScalar distance,int lifeTime,const btVector3& color) {
	//std::cout << "drawcontactpoint" << std::endl;
}
      
void DebugDrawer::reportErrorWarning(const char* warningString) {
	//std::cout << "reporterrorwarning" << std::endl;
}
      
void DebugDrawer::draw3dText(const btVector3& location,const char* textString) {
	//std::cout << "draw3dtext" << std::endl;
}
      
void DebugDrawer::setDebugMode(int newDebugMode) {
	debugMode = newDebugMode;
}

void DebugDrawer::drawPoint(glm::vec3 p, glm::mat4 position) {
	wireframeShader->useShader();


	float vertices_position[6] = {
	p.x, p.y, p.z, 10, 10, 10
};
	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, sizeof(vertices_position), vertices_position, GL_STATIC_DRAW);

	glUniformMatrix4fv(
		glGetUniformLocation(wireframeShader->programHandle, "projectionMatrix"), 1,
		GL_FALSE, glm::value_ptr(camera->matrix()));
	glBindVertexArray(vao);

	glUniformMatrix4fv(glGetUniformLocation(wireframeShader->programHandle, "modelMatrix"), 1, GL_FALSE, glm::value_ptr(position));
	glPointSize(6.0);
	
	glDrawArrays(GL_POINTS, 0, 2);
}