#version 330

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in int bone_id;

out vec3 worldNormal;
out vec2 fUVs;
out vec3 fragPosition;
out vec4 SM_tex_coord;
out vec2 velocity;

uniform mat4 projectionMatrix;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 depthBiasVP;
uniform mat4 act_blurVP;
uniform mat4 prev_blurVP;

uniform sampler2D blurmap;

void main(){


	vec4 eye_position = viewMatrix * modelMatrix * vec4(position, 1.0);
	gl_Position = projectionMatrix * modelMatrix * vec4(position,1);

    fragPosition = eye_position.xyz;
    worldNormal   = (viewMatrix * modelMatrix  * vec4(normal, 0.0)).xyz;
    fUVs = uv;
	
	SM_tex_coord = depthBiasVP * modelMatrix * vec4(position,1);


	//calculate blur velocity vector 
   // Get the depth buffer value at this pixel.  
   float depthblur = texture(blurmap, uv).r;
   //float zOverW = tex2D(depthTexture, texCoord);  
   // H is the viewport position at this pixel in the range -1 to 1.  
   vec4 H = vec4(uv.x * 2 - 1, (1 - uv.y) * 2 - 1, depthblur, 1);  
  // Transform by the view-projection inverse.  
   vec4 D =  inverse (act_blurVP)* H;  
   // Divide by w to get the world position.  
   vec4 worldPos = D / D.w; 

   //calculate the previous position
   // Current viewport position  
   vec4 currentPos = H;  
   // Use the world position, and transform by the previous view-projection matrix. 
   vec4 previousPos = prev_blurVP  * worldPos;  
   // Convert to nonhomogeneous points [-1,1] by dividing by w.  
   previousPos = previousPos / previousPos.w;  
   // Use this frame's position and last frame's to compute the pixel  
   // velocity.  
   velocity = (currentPos.xy - previousPos.xy)/2.0;  

}